@@ -15,13 +15,17 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Task APIs.."""
+"""Task APIs."""
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
 from deprecated import deprecated
 
 from airflow.api.common.experimental import check_and_get_dag
-from airflow.models import TaskInstance
+
+if TYPE_CHECKING:
+    from airflow.models import TaskInstance
 
 
 @deprecated(reason="Use DAG().get_task", version="2.2.4")
