@@ -48,8 +48,7 @@ class TimeDeltaSchema(Schema):
     @marshmallow.post_load
     def make_time_delta(self, data, **kwargs):
         """Create time delta based on data."""
-        if "objectType" in data:
-            del data["objectType"]
+        data.pop("objectType", None)
         return datetime.timedelta(**data)
 
 
@@ -76,9 +75,7 @@ class RelativeDeltaSchema(Schema):
     @marshmallow.post_load
     def make_relative_delta(self, data, **kwargs):
         """Create relative delta based on data."""
-        if "objectType" in data:
-            del data["objectType"]
-
+        data.pop("objectType", None)
         return relativedelta.relativedelta(**data)
 
 
