@@ -30,35 +30,35 @@ warnings.warn(
 )
 
 __deprecated_classes = {
-    'gcp_field_sanitizer': {
-        'GcpBodyFieldSanitizer': 'airflow.providers.google.cloud.utils.field_sanitizer.GcpBodyFieldSanitizer',
-        'GcpFieldSanitizerException': (
-            'airflow.providers.google.cloud.utils.field_sanitizer.GcpFieldSanitizerException'
+    "gcp_field_sanitizer": {
+        "GcpBodyFieldSanitizer": "airflow.providers.google.cloud.utils.field_sanitizer.GcpBodyFieldSanitizer",
+        "GcpFieldSanitizerException": (
+            "airflow.providers.google.cloud.utils.field_sanitizer.GcpFieldSanitizerException"
         ),
     },
-    'gcp_field_validator': {
-        'GcpBodyFieldValidator': 'airflow.providers.google.cloud.utils.field_validator.GcpBodyFieldValidator',
-        'GcpFieldValidationException': (
-            'airflow.providers.google.cloud.utils.field_validator.GcpFieldValidationException'
+    "gcp_field_validator": {
+        "GcpBodyFieldValidator": "airflow.providers.google.cloud.utils.field_validator.GcpBodyFieldValidator",
+        "GcpFieldValidationException": (
+            "airflow.providers.google.cloud.utils.field_validator.GcpFieldValidationException"
         ),
-        'GcpValidationSpecificationException': (
-            'airflow.providers.google.cloud.utils.field_validator.GcpValidationSpecificationException'
+        "GcpValidationSpecificationException": (
+            "airflow.providers.google.cloud.utils.field_validator.GcpValidationSpecificationException"
         ),
     },
-    'mlengine_operator_utils': {
-        'create_evaluate_ops': (
-            'airflow.providers.google.cloud.utils.mlengine_operator_utils.create_evaluate_ops'
+    "mlengine_operator_utils": {
+        "create_evaluate_ops": (
+            "airflow.providers.google.cloud.utils.mlengine_operator_utils.create_evaluate_ops"
         ),
     },
-    'mlengine_prediction_summary': {
-        'JsonCoder': 'airflow.providers.google.cloud.utils.mlengine_prediction_summary.JsonCoder',
-        'MakeSummary': 'airflow.providers.google.cloud.utils.mlengine_prediction_summary.MakeSummary',
+    "mlengine_prediction_summary": {
+        "JsonCoder": "airflow.providers.google.cloud.utils.mlengine_prediction_summary.JsonCoder",
+        "MakeSummary": "airflow.providers.google.cloud.utils.mlengine_prediction_summary.MakeSummary",
     },
-    'sendgrid': {
-        'import_string': 'airflow.utils.module_loading.import_string',
+    "sendgrid": {
+        "import_string": "airflow.utils.module_loading.import_string",
     },
-    'weekday': {
-        'WeekDay': 'airflow.utils.weekday.WeekDay',
+    "weekday": {
+        "WeekDay": "airflow.utils.weekday.WeekDay",
     },
 }
 
