@@ -16,47 +16,20 @@
 # under the License.
 from __future__ import annotations
 
-from typing import Callable, Sequence
+from typing import TYPE_CHECKING, Callable
 
-from airflow.decorators.base import DecoratedOperator, TaskDecorator, task_decorator_factory
+from airflow.decorators.base import task_decorator_factory
+from airflow.decorators.python import _PythonDecoratedOperator
 from airflow.operators.python import ShortCircuitOperator
 
+if TYPE_CHECKING:
+    from airflow.decorators.base import TaskDecorator
 
-class _ShortCircuitDecoratedOperator(DecoratedOperator, ShortCircuitOperator):
-    """
-    Wraps a Python callable and captures args/kwargs when called for execution.
 
-    :param python_callable: A reference to an object that is callable
-    :param op_kwargs: a dictionary of keyword arguments that will get unpacked
-        in your function (templated)
-    :param op_args: a list of positional arguments that will get unpacked when
-        calling your callable (templated)
-    :param multiple_outputs: If set to True, the decorated function's return value will be unrolled to
-        multiple XCom values. Dict will unroll to XCom values with its keys as XCom keys. Defaults to False.
-    """
+class _ShortCircuitDecoratedOperator(_PythonDecoratedOperator, ShortCircuitOperator):
+    """Wraps a Python callable and captures args/kwargs when called for execution."""
 
-    template_fields: Sequence[str] = ('op_args', 'op_kwargs')
-    template_fields_renderers = {"op_args": "py", "op_kwargs": "py"}
-
-    # since we won't mutate the arguments, we should just do the shallow copy
-    # there are some cases we can't deepcopy the objects (e.g protobuf).
-    shallow_copy_attrs: Sequence[str] = ('python_callable',)
-
-    custom_operator_name: str = '@task.short_circuit'
-
-    def __init__(self, *, python_callable, op_args, op_kwargs, **kwargs) -> None:
-        kwargs_to_upstream = {
-            "python_callable": python_callable,
-            "op_args": op_args,
-            "op_kwargs": op_kwargs,
-        }
-        super().__init__(
-            kwargs_to_upstream=kwargs_to_upstream,
-            python_callable=python_callable,
-            op_args=op_args,
-            op_kwargs=op_kwargs,
-            **kwargs,
-        )
+    custom_operator_name: str = "@task.short_circuit"
 
 
 def short_circuit_task(
@@ -64,7 +37,8 @@ def short_circuit_task(
     multiple_outputs: bool | None = None,
     **kwargs,
 ) -> TaskDecorator:
-    """Wraps a function into an ShortCircuitOperator.
+    """
+    Wrap a function into an ShortCircuitOperator.
 
     Accepts kwargs for operator kwarg. Can be reused in a single DAG.
 
