@@ -19,18 +19,16 @@ from __future__ import annotations
 
 import collections.abc
 import contextlib
-import datetime
 import inspect
 import itertools
 import json
 import logging
 import pickle
 import warnings
-from functools import wraps
+from functools import cached_property, wraps
 from typing import TYPE_CHECKING, Any, Generator, Iterable, cast, overload
 
 import attr
-import pendulum
 from sqlalchemy import (
     Column,
     ForeignKeyConstraint,
@@ -39,14 +37,15 @@ from sqlalchemy import (
     LargeBinary,
     PrimaryKeyConstraint,
     String,
+    delete,
     text,
 )
 from sqlalchemy.ext.associationproxy import association_proxy
-from sqlalchemy.orm import Query, Session, reconstructor, relationship
+from sqlalchemy.orm import Query, reconstructor, relationship
 from sqlalchemy.orm.exc import NoResultFound
 
 from airflow import settings
-from airflow.compat.functools import cached_property
+from airflow.api_internal.internal_api_call import internal_api_call
 from airflow.configuration import conf
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.models.base import COLLATION_ARGS, ID_LEN, Base
@@ -57,15 +56,22 @@ from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.sqlalchemy import UtcDateTime
 
+# XCom constants below are needed for providers backward compatibility,
+# which should import the constants directly after apache-airflow>=2.6.0
+from airflow.utils.xcom import (
+    MAX_XCOM_SIZE,  # noqa: F401
+    XCOM_RETURN_KEY,
+)
+
 log = logging.getLogger(__name__)
 
-# MAX XCOM Size is 48KB
-# https://github.com/apache/airflow/pull/1618#discussion_r68249677
-MAX_XCOM_SIZE = 49344
-XCOM_RETURN_KEY = "return_value"
-
 if TYPE_CHECKING:
-    from airflow.models.taskinstance import TaskInstanceKey
+    import datetime
+
+    import pendulum
+    from sqlalchemy.orm import Session
+
+    from airflow.models.taskinstancekey import TaskInstanceKey
 
 
 class BaseXCom(Base, LoggingMixin):
@@ -119,7 +125,8 @@ class BaseXCom(Base, LoggingMixin):
     @reconstructor
     def init_on_load(self):
         """
-        Called by the ORM after the instance has been loaded from the DB or otherwise reconstituted
+        Execute after the instance has been loaded from the DB or otherwise reconstituted; called by the ORM.
+
         i.e automatically deserialize Xcom value when loading from DB.
         """
         self.value = self.orm_deserialize_value()
@@ -169,7 +176,10 @@ class BaseXCom(Base, LoggingMixin):
         execution_date: datetime.datetime,
         session: Session = NEW_SESSION,
     ) -> None:
-        """:sphinx-autoapi-skip:"""
+        """Store an XCom value.
+
+        :sphinx-autoapi-skip:
+        """
 
     @classmethod
     @provide_session
@@ -185,7 +195,10 @@ class BaseXCom(Base, LoggingMixin):
         run_id: str | None = None,
         map_index: int = -1,
     ) -> None:
-        """:sphinx-autoapi-skip:"""
+        """Store an XCom value.
+
+        :sphinx-autoapi-skip:
+        """
         from airflow.models.dagrun import DagRun
 
         if not exactly_one(execution_date is not None, run_id is not None):
@@ -241,13 +254,15 @@ class BaseXCom(Base, LoggingMixin):
         )
 
         # Remove duplicate XComs and insert a new one.
-        session.query(cls).filter(
-            cls.key == key,
-            cls.run_id == run_id,
-            cls.task_id == task_id,
-            cls.dag_id == dag_id,
-            cls.map_index == map_index,
-        ).delete()
+        session.execute(
+            delete(cls).where(
+                cls.key == key,
+                cls.run_id == run_id,
+                cls.task_id == task_id,
+                cls.dag_id == dag_id,
+                cls.map_index == map_index,
+            )
+        )
         new = cast(Any, cls)(  # Work around Mypy complaining model not defining '__init__'.
             dag_run_id=dag_run_id,
             key=key,
@@ -260,10 +275,10 @@ class BaseXCom(Base, LoggingMixin):
         session.add(new)
         session.flush()
 
-    @classmethod
+    @staticmethod
     @provide_session
+    @internal_api_call
     def get_value(
-        cls,
         *,
         ti_key: TaskInstanceKey,
         key: str | None = None,
@@ -284,7 +299,7 @@ class BaseXCom(Base, LoggingMixin):
         :param session: Database session. If not given, a new session will be
             created for this function.
         """
-        return cls.get_one(
+        return BaseXCom.get_one(
             key=key,
             task_id=ti_key.task_id,
             dag_id=ti_key.dag_id,
@@ -294,9 +309,9 @@ class BaseXCom(Base, LoggingMixin):
         )
 
     @overload
-    @classmethod
+    @staticmethod
+    @internal_api_call
     def get_one(
-        cls,
         *,
         key: str | None = None,
         dag_id: str | None = None,
@@ -337,9 +352,9 @@ class BaseXCom(Base, LoggingMixin):
         """
 
     @overload
-    @classmethod
+    @staticmethod
+    @internal_api_call
     def get_one(
-        cls,
         execution_date: datetime.datetime,
         key: str | None = None,
         task_id: str | None = None,
@@ -347,12 +362,15 @@ class BaseXCom(Base, LoggingMixin):
         include_prior_dates: bool = False,
         session: Session = NEW_SESSION,
     ) -> Any | None:
-        """:sphinx-autoapi-skip:"""
+        """Retrieve an XCom value, optionally meeting certain criteria.
 
-    @classmethod
+        :sphinx-autoapi-skip:
+        """
+
+    @staticmethod
     @provide_session
+    @internal_api_call
     def get_one(
-        cls,
         execution_date: datetime.datetime | None = None,
         key: str | None = None,
         task_id: str | None = None,
@@ -363,12 +381,15 @@ class BaseXCom(Base, LoggingMixin):
         run_id: str | None = None,
         map_index: int | None = None,
     ) -> Any | None:
-        """:sphinx-autoapi-skip:"""
+        """Retrieve an XCom value, optionally meeting certain criteria.
+
+        :sphinx-autoapi-skip:
+        """
         if not exactly_one(execution_date is not None, run_id is not None):
             raise ValueError("Exactly one of run_id or execution_date must be passed")
 
         if run_id:
-            query = cls.get_many(
+            query = BaseXCom.get_many(
                 run_id=run_id,
                 key=key,
                 task_ids=task_id,
@@ -384,7 +405,7 @@ class BaseXCom(Base, LoggingMixin):
 
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", RemovedInAirflow3Warning)
-                query = cls.get_many(
+                query = BaseXCom.get_many(
                     execution_date=execution_date,
                     key=key,
                     task_ids=task_id,
@@ -397,15 +418,14 @@ class BaseXCom(Base, LoggingMixin):
         else:
             raise RuntimeError("Should not happen?")
 
-        result = query.with_entities(cls.value).first()
+        result = query.with_entities(BaseXCom.value).first()
         if result:
-            return cls.deserialize_value(result)
+            return BaseXCom.deserialize_value(result)
         return None
 
     @overload
-    @classmethod
+    @staticmethod
     def get_many(
-        cls,
         *,
         run_id: str,
         key: str | None = None,
@@ -438,12 +458,13 @@ class BaseXCom(Base, LoggingMixin):
             returned regardless of the run it belongs to.
         :param session: Database session. If not given, a new session will be
             created for this function.
+        :param limit: Limiting returning XComs
         """
 
     @overload
-    @classmethod
+    @staticmethod
+    @internal_api_call
     def get_many(
-        cls,
         execution_date: datetime.datetime,
         key: str | None = None,
         task_ids: str | Iterable[str] | None = None,
@@ -453,12 +474,15 @@ class BaseXCom(Base, LoggingMixin):
         limit: int | None = None,
         session: Session = NEW_SESSION,
     ) -> Query:
-        """:sphinx-autoapi-skip:"""
+        """Composes a query to get one or more XCom entries.
 
-    @classmethod
+        :sphinx-autoapi-skip:
+        """
+
+    @staticmethod
     @provide_session
+    @internal_api_call
     def get_many(
-        cls,
         execution_date: datetime.datetime | None = None,
         key: str | None = None,
         task_ids: str | Iterable[str] | None = None,
@@ -470,7 +494,10 @@ class BaseXCom(Base, LoggingMixin):
         *,
         run_id: str | None = None,
     ) -> Query:
-        """:sphinx-autoapi-skip:"""
+        """Composes a query to get one or more XCom entries.
+
+        :sphinx-autoapi-skip:
+        """
         from airflow.models.dagrun import DagRun
 
         if not exactly_one(execution_date is not None, run_id is not None):
@@ -482,40 +509,42 @@ class BaseXCom(Base, LoggingMixin):
             message = "Passing 'execution_date' to 'XCom.get_many()' is deprecated. Use 'run_id' instead."
             warnings.warn(message, RemovedInAirflow3Warning, stacklevel=3)
 
-        query = session.query(cls).join(cls.dag_run)
+        query = session.query(BaseXCom).join(BaseXCom.dag_run)
 
         if key:
-            query = query.filter(cls.key == key)
+            query = query.filter(BaseXCom.key == key)
 
         if is_container(task_ids):
-            query = query.filter(cls.task_id.in_(task_ids))
+            query = query.filter(BaseXCom.task_id.in_(task_ids))
         elif task_ids is not None:
-            query = query.filter(cls.task_id == task_ids)
+            query = query.filter(BaseXCom.task_id == task_ids)
 
         if is_container(dag_ids):
-            query = query.filter(cls.dag_id.in_(dag_ids))
+            query = query.filter(BaseXCom.dag_id.in_(dag_ids))
         elif dag_ids is not None:
-            query = query.filter(cls.dag_id == dag_ids)
+            query = query.filter(BaseXCom.dag_id == dag_ids)
 
         if isinstance(map_indexes, range) and map_indexes.step == 1:
-            query = query.filter(cls.map_index >= map_indexes.start, cls.map_index < map_indexes.stop)
+            query = query.filter(
+                BaseXCom.map_index >= map_indexes.start, BaseXCom.map_index < map_indexes.stop
+            )
         elif is_container(map_indexes):
-            query = query.filter(cls.map_index.in_(map_indexes))
+            query = query.filter(BaseXCom.map_index.in_(map_indexes))
         elif map_indexes is not None:
-            query = query.filter(cls.map_index == map_indexes)
+            query = query.filter(BaseXCom.map_index == map_indexes)
 
         if include_prior_dates:
             if execution_date is not None:
                 query = query.filter(DagRun.execution_date <= execution_date)
             else:
                 dr = session.query(DagRun.execution_date).filter(DagRun.run_id == run_id).subquery()
-                query = query.filter(cls.execution_date <= dr.c.execution_date)
+                query = query.filter(BaseXCom.execution_date <= dr.c.execution_date)
         elif execution_date is not None:
             query = query.filter(DagRun.execution_date == execution_date)
         else:
-            query = query.filter(cls.run_id == run_id)
+            query = query.filter(BaseXCom.run_id == run_id)
 
-        query = query.order_by(DagRun.execution_date.desc(), cls.timestamp.desc())
+        query = query.order_by(DagRun.execution_date.desc(), BaseXCom.timestamp.desc())
         if limit:
             return query.limit(limit)
         return query
@@ -533,9 +562,9 @@ class BaseXCom(Base, LoggingMixin):
         session.commit()
 
     @overload
-    @classmethod
+    @staticmethod
+    @internal_api_call
     def clear(
-        cls,
         *,
         dag_id: str,
         task_id: str,
@@ -558,20 +587,23 @@ class BaseXCom(Base, LoggingMixin):
         """
 
     @overload
-    @classmethod
+    @staticmethod
+    @internal_api_call
     def clear(
-        cls,
         execution_date: pendulum.DateTime,
         dag_id: str,
         task_id: str,
         session: Session = NEW_SESSION,
     ) -> None:
-        """:sphinx-autoapi-skip:"""
+        """Clear all XCom data from the database for the given task instance.
 
-    @classmethod
+        :sphinx-autoapi-skip:
+        """
+
+    @staticmethod
     @provide_session
+    @internal_api_call
     def clear(
-        cls,
         execution_date: pendulum.DateTime | None = None,
         dag_id: str | None = None,
         task_id: str | None = None,
@@ -580,7 +612,10 @@ class BaseXCom(Base, LoggingMixin):
         run_id: str | None = None,
         map_index: int | None = None,
     ) -> None:
-        """:sphinx-autoapi-skip:"""
+        """Clear all XCom data from the database for the given task instance.
+
+        :sphinx-autoapi-skip:
+        """
         from airflow.models import DagRun
 
         # Given the historic order of this function (execution_date was first argument) to add a new optional
@@ -605,7 +640,7 @@ class BaseXCom(Base, LoggingMixin):
                 .scalar()
             )
 
-        query = session.query(cls).filter_by(dag_id=dag_id, task_id=task_id, run_id=run_id)
+        query = session.query(BaseXCom).filter_by(dag_id=dag_id, task_id=task_id, run_id=run_id)
         if map_index is not None:
             query = query.filter_by(map_index=map_index)
         query.delete()
@@ -657,7 +692,7 @@ class BaseXCom(Base, LoggingMixin):
 
     @staticmethod
     def deserialize_value(result: XCom) -> Any:
-        """Deserialize XCom value from str or pickle object"""
+        """Deserialize XCom value from str or pickle object."""
         return BaseXCom._deserialize_value(result, False)
 
     def orm_deserialize_value(self) -> Any:
@@ -806,7 +841,7 @@ def _patch_outdated_serializer(clazz: type[BaseXCom], params: Iterable[str]) ->
 
 def _get_function_params(function) -> list[str]:
     """
-    Returns the list of variables names of a function
+    Return the list of variables names of a function.
 
     :param function: The function to inspect
     """
@@ -818,10 +853,10 @@ def _get_function_params(function) -> list[str]:
 
 
 def resolve_xcom_backend() -> type[BaseXCom]:
-    """Resolves custom XCom class
+    """Resolve custom XCom class.
 
-    Confirms that custom XCom class extends the BaseXCom.
-    Compares the function signature of the custom XCom serialize_value to the base XCom serialize_value.
+    Confirm that custom XCom class extends the BaseXCom.
+    Compare the function signature of the custom XCom serialize_value to the base XCom serialize_value.
     """
     clazz = conf.getimport("core", "xcom_backend", fallback=f"airflow.models.xcom.{BaseXCom.__name__}")
     if not clazz:
@@ -832,7 +867,7 @@ def resolve_xcom_backend() -> type[BaseXCom]:
         )
     base_xcom_params = _get_function_params(BaseXCom.serialize_value)
     xcom_params = _get_function_params(clazz.serialize_value)
-    if not set(base_xcom_params) == set(xcom_params):
+    if set(base_xcom_params) != set(xcom_params):
         _patch_outdated_serializer(clazz=clazz, params=xcom_params)
     return clazz
 
