@@ -19,27 +19,32 @@ from __future__ import annotations
 
 import contextlib
 import inspect
-from typing import TYPE_CHECKING, Any, Callable, Iterator, Mapping, Sequence, Union, overload
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Iterator, Mapping, Sequence, Union, overload
 
-from sqlalchemy import func, or_
-from sqlalchemy.orm import Session
+from sqlalchemy import func, or_, select
 
 from airflow.exceptions import AirflowException, XComNotFound
 from airflow.models.abstractoperator import AbstractOperator
 from airflow.models.mappedoperator import MappedOperator
-from airflow.models.taskmixin import DAGNode, DependencyMixin
-from airflow.utils.context import Context
-from airflow.utils.edgemodifier import EdgeModifier
+from airflow.models.taskmixin import DependencyMixin
+from airflow.utils.db import exists_query
 from airflow.utils.mixins import ResolveMixin
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.setup_teardown import SetupTeardownContext
 from airflow.utils.state import State
+from airflow.utils.trigger_rule import TriggerRule
 from airflow.utils.types import NOTSET, ArgNotSet
 from airflow.utils.xcom import XCOM_RETURN_KEY
 
 if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
+    from airflow.models.baseoperator import BaseOperator
     from airflow.models.dag import DAG
     from airflow.models.operator import Operator
+    from airflow.models.taskmixin import DAGNode
+    from airflow.utils.context import Context
+    from airflow.utils.edgemodifier import EdgeModifier
 
 # Callable objects contained by MapXComArg. We only accept callables from
 # the user, but deserialize them into strings in a serialized XComArg for
@@ -82,11 +87,11 @@ class XComArg(ResolveMixin, DependencyMixin):
 
     @overload
     def __new__(cls: type[XComArg], operator: Operator, key: str = XCOM_RETURN_KEY) -> XComArg:
-        """Called when the user writes ``XComArg(...)`` directly."""
+        """Execute when the user writes ``XComArg(...)`` directly."""
 
     @overload
     def __new__(cls: type[XComArg]) -> XComArg:
-        """Called by Python internals from subclasses."""
+        """Execute by Python internals from subclasses."""
 
     def __new__(cls, *args, **kwargs) -> XComArg:
         if cls is XComArg:
@@ -125,12 +130,12 @@ class XComArg(ResolveMixin, DependencyMixin):
 
     @property
     def roots(self) -> list[DAGNode]:
-        """Required by TaskMixin"""
+        """Required by TaskMixin."""
         return [op for op, _ in self.iter_references()]
 
     @property
     def leaves(self) -> list[DAGNode]:
-        """Required by TaskMixin"""
+        """Required by TaskMixin."""
         return [op for op, _ in self.iter_references()]
 
     def set_upstream(
@@ -152,7 +157,8 @@ class XComArg(ResolveMixin, DependencyMixin):
             operator.set_downstream(task_or_task_list, edge_modifier)
 
     def _serialize(self) -> dict[str, Any]:
-        """Called by DAG serialization.
+        """
+        Serialize a DAG.
 
         The implementation should be the inverse function to ``deserialize``,
         returning a data dict converted from this XComArg derivative. DAG
@@ -164,7 +170,8 @@ class XComArg(ResolveMixin, DependencyMixin):
 
     @classmethod
     def _deserialize(cls, data: dict[str, Any], dag: DAG) -> XComArg:
-        """Called when deserializing a DAG.
+        """
+        Deserialize a DAG.
 
         The implementation should be the inverse function to ``serialize``,
         implementing given a data dict converted from this XComArg derivative,
@@ -206,10 +213,10 @@ class XComArg(ResolveMixin, DependencyMixin):
         raise NotImplementedError()
 
     def __enter__(self):
-        if not self.operator._is_setup and not self.operator._is_teardown:
+        if not self.operator.is_setup and not self.operator.is_teardown:
             raise AirflowException("Only setup/teardown tasks can be used as context managers.")
         SetupTeardownContext.push_setup_teardown_task(self.operator)
-        return self
+        return SetupTeardownContext
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         SetupTeardownContext.set_work_task_roots_and_leaves()
@@ -243,7 +250,7 @@ class PlainXComArg(XComArg):
         return self.operator == other.operator and self.key == other.key
 
     def __getitem__(self, item: str) -> XComArg:
-        """Implements xcomresult['some_result_key']"""
+        """Implement xcomresult['some_result_key']."""
         if not isinstance(item, str):
             raise ValueError(f"XComArg only supports str lookup, received {type(item).__name__}")
         return PlainXComArg(operator=self.operator, key=item)
@@ -269,7 +276,7 @@ class PlainXComArg(XComArg):
 
     def __str__(self) -> str:
         """
-        Backward compatibility for old-style jinja used in Airflow Operators
+        Backward compatibility for old-style jinja used in Airflow Operators.
 
         **Example**: to use XComArg at BashOperator::
 
@@ -296,6 +303,53 @@ class PlainXComArg(XComArg):
     def _deserialize(cls, data: dict[str, Any], dag: DAG) -> XComArg:
         return cls(dag.get_task(data["task_id"]), data["key"])
 
+    @property
+    def is_setup(self) -> bool:
+        return self.operator.is_setup
+
+    @is_setup.setter
+    def is_setup(self, val: bool):
+        self.operator.is_setup = val
+
+    @property
+    def is_teardown(self) -> bool:
+        return self.operator.is_teardown
+
+    @is_teardown.setter
+    def is_teardown(self, val: bool):
+        self.operator.is_teardown = val
+
+    @property
+    def on_failure_fail_dagrun(self) -> bool:
+        return self.operator.on_failure_fail_dagrun
+
+    @on_failure_fail_dagrun.setter
+    def on_failure_fail_dagrun(self, val: bool):
+        self.operator.on_failure_fail_dagrun = val
+
+    def as_setup(self) -> DependencyMixin:
+        for operator, _ in self.iter_references():
+            operator.is_setup = True
+        return self
+
+    def as_teardown(
+        self,
+        *,
+        setups: BaseOperator | Iterable[BaseOperator] | ArgNotSet = NOTSET,
+        on_failure_fail_dagrun=NOTSET,
+    ):
+        for operator, _ in self.iter_references():
+            operator.is_teardown = True
+            operator.trigger_rule = TriggerRule.ALL_DONE_SETUP_SUCCESS
+            if on_failure_fail_dagrun is not NOTSET:
+                operator.on_failure_fail_dagrun = on_failure_fail_dagrun
+            if not isinstance(setups, ArgNotSet):
+                setups = [setups] if isinstance(setups, DependencyMixin) else setups
+                for s in setups:
+                    s.is_setup = True
+                    s >> operator
+        return self
+
     def iter_references(self) -> Iterator[tuple[Operator, str]]:
         yield self.operator, self.key
 
@@ -316,7 +370,7 @@ class PlainXComArg(XComArg):
 
         task = self.operator
         if isinstance(task, MappedOperator):
-            unfinished_ti_count_query = session.query(func.count(TaskInstance.map_index)).filter(
+            unfinished_ti_exists = exists_query(
                 TaskInstance.dag_id == task.dag_id,
                 TaskInstance.run_id == run_id,
                 TaskInstance.task_id == task.task_id,
@@ -327,10 +381,11 @@ class PlainXComArg(XComArg):
                     TaskInstance.state.is_(None),
                     TaskInstance.state.in_(s.value for s in State.unfinished if s is not None),
                 ),
+                session=session,
             )
-            if unfinished_ti_count_query.scalar():
+            if unfinished_ti_exists:
                 return None  # Not all of the expanded tis are done yet.
-            query = session.query(func.count(XCom.map_index)).filter(
+            query = select(func.count(XCom.map_index)).where(
                 XCom.dag_id == task.dag_id,
                 XCom.run_id == run_id,
                 XCom.task_id == task.task_id,
@@ -338,13 +393,13 @@ class PlainXComArg(XComArg):
                 XCom.key == XCOM_RETURN_KEY,
             )
         else:
-            query = session.query(TaskMap.length).filter(
+            query = select(TaskMap.length).where(
                 TaskMap.dag_id == task.dag_id,
                 TaskMap.run_id == run_id,
                 TaskMap.task_id == task.task_id,
                 TaskMap.map_index < 0,
             )
-        return query.scalar()
+        return session.scalar(query)
 
     @provide_session
     def resolve(self, context: Context, session: Session = NEW_SESSION) -> Any:
