@@ -21,14 +21,16 @@ import os
 import shutil
 import warnings
 from functools import cached_property
-from typing import Container, Sequence
+from typing import TYPE_CHECKING, Container, Sequence
 
 from airflow.exceptions import AirflowException, AirflowSkipException
 from airflow.hooks.subprocess import SubprocessHook
 from airflow.models.baseoperator import BaseOperator
-from airflow.utils.context import Context
 from airflow.utils.operator_helpers import context_to_airflow_vars
 
+if TYPE_CHECKING:
+    from airflow.utils.context import Context
+
 
 class BashOperator(BaseOperator):
     r"""
