@@ -15,11 +15,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""
-This module contains an operator to run downstream tasks only for the
-latest scheduled DagRun
-"""
-from typing import TYPE_CHECKING, Dict, Iterable, Union
+"""Contains an operator to run downstream tasks only for the latest scheduled DagRun."""
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Iterable
 
 import pendulum
 
@@ -27,51 +26,51 @@ from airflow.operators.branch import BaseBranchOperator
 
 if TYPE_CHECKING:
     from airflow.models import DAG, DagRun
+    from airflow.utils.context import Context
 
 
 class LatestOnlyOperator(BaseBranchOperator):
     """
-    Allows a workflow to skip tasks that are not running during the most
-    recent schedule interval.
+    Skip tasks that are not running during the most recent schedule interval.
 
-    If the task is run outside of the latest schedule interval (i.e. external_trigger),
+    If the task is run outside the latest schedule interval (i.e. external_trigger),
     all directly downstream tasks will be skipped.
 
     Note that downstream tasks are never skipped if the given DAG_Run is
     marked as externally triggered.
     """
 
-    ui_color = '#e9ffdb'  # nyanza
+    ui_color = "#e9ffdb"  # nyanza
 
-    def choose_branch(self, context: Dict) -> Union[str, Iterable[str]]:
+    def choose_branch(self, context: Context) -> str | Iterable[str]:
         # If the DAG Run is externally triggered, then return without
         # skipping downstream tasks
-        dag_run: "DagRun" = context["dag_run"]
+        dag_run: DagRun = context["dag_run"]
         if dag_run.external_trigger:
             self.log.info("Externally triggered DAG_Run: allowing execution to proceed.")
-            return list(context['task'].get_direct_relative_ids(upstream=False))
+            return list(context["task"].get_direct_relative_ids(upstream=False))
 
-        dag: "DAG" = context["dag"]
+        dag: DAG = context["dag"]
         next_info = dag.next_dagrun_info(dag.get_run_data_interval(dag_run), restricted=False)
-        now = pendulum.now('UTC')
+        now = pendulum.now("UTC")
 
         if next_info is None:
             self.log.info("Last scheduled execution: allowing execution to proceed.")
-            return list(context['task'].get_direct_relative_ids(upstream=False))
+            return list(context["task"].get_direct_relative_ids(upstream=False))
 
         left_window, right_window = next_info.data_interval
         self.log.info(
-            'Checking latest only with left_window: %s right_window: %s now: %s',
+            "Checking latest only with left_window: %s right_window: %s now: %s",
             left_window,
             right_window,
             now,
         )
 
         if not left_window < now <= right_window:
-            self.log.info('Not latest execution, skipping downstream.')
+            self.log.info("Not latest execution, skipping downstream.")
             # we return an empty list, thus the parent BaseBranchOperator
             # won't exclude any downstream tasks from skipping.
             return []
         else:
-            self.log.info('Latest, allowing execution to proceed.')
-            return list(context['task'].get_direct_relative_ids(upstream=False))
+            self.log.info("Latest, allowing execution to proceed.")
+            return list(context["task"].get_direct_relative_ids(upstream=False))
