@@ -17,18 +17,16 @@
 # under the License.
 from __future__ import annotations
 
-import collections
 import collections.abc
 import functools
+from collections import Counter
 from typing import TYPE_CHECKING, Iterator, KeysView, NamedTuple
 
 from sqlalchemy import and_, func, or_, select
 
 from airflow.models.taskinstance import PAST_DEPENDS_MET
-from airflow.ti_deps.dep_context import DepContext
-from airflow.ti_deps.deps.base_ti_dep import BaseTIDep, TIDepStatus
+from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
 from airflow.utils.state import TaskInstanceState
-from airflow.utils.task_group import MappedTaskGroup
 from airflow.utils.trigger_rule import TriggerRule as TR
 
 if TYPE_CHECKING:
@@ -37,6 +35,8 @@ if TYPE_CHECKING:
 
     from airflow import DAG
     from airflow.models.taskinstance import TaskInstance
+    from airflow.ti_deps.dep_context import DepContext
+    from airflow.ti_deps.deps.base_ti_dep import TIDepStatus
 
 
 class _UpstreamTIStates(NamedTuple):
@@ -64,8 +64,8 @@ class _UpstreamTIStates(NamedTuple):
         :param ti: the ti that we want to calculate deps for
         :param finished_tis: all the finished tasks of the dag_run
         """
-        counter: dict[str, int] = collections.Counter()
-        setup_counter: dict[str, int] = collections.Counter()
+        counter: dict[str, int] = Counter()
+        setup_counter: dict[str, int] = Counter()
         for ti in finished_upstreams:
             curr_state = {ti.state: 1}
             counter.update(curr_state)
@@ -132,20 +132,6 @@ class TriggerRuleDep(BaseTIDep):
             """
             return ti.task.get_mapped_ti_count(ti.run_id, session=session)
 
-        def _iter_expansion_dependencies() -> Iterator[str]:
-            from airflow.models.mappedoperator import MappedOperator
-
-            if isinstance(ti.task, MappedOperator):
-                for op in ti.task.iter_mapped_dependencies():
-                    yield op.task_id
-            task_group = ti.task.task_group
-            if task_group and task_group.iter_mapped_task_groups():
-                yield from (
-                    op.task_id
-                    for tg in task_group.iter_mapped_task_groups()
-                    for op in tg.iter_mapped_dependencies()
-                )
-
         @functools.lru_cache
         def _get_relevant_upstream_map_indexes(upstream_id: str) -> int | range | None:
             """Get the given task's map indexes relevant to the current ti.
@@ -156,9 +142,6 @@ class TriggerRuleDep(BaseTIDep):
             """
             if TYPE_CHECKING:
                 assert isinstance(ti.task.dag, DAG)
-            if isinstance(ti.task.task_group, MappedTaskGroup):
-                if upstream_id not in set(_iter_expansion_dependencies()):
-                    return None
             try:
                 expanded_ti_count = _get_expanded_ti_count()
             except (NotFullyPopulated, NotMapped):
@@ -211,7 +194,7 @@ class TriggerRuleDep(BaseTIDep):
                 return
             # Otherwise we need to figure out which map indexes are depended on
             # for each upstream by the current task instance.
-            for upstream_id in relevant_tasks.keys():
+            for upstream_id in relevant_tasks:
                 map_indexes = _get_relevant_upstream_map_indexes(upstream_id)
                 if map_indexes is None:  # All tis of this upstream are dependencies.
                     yield (TaskInstance.task_id == upstream_id)
@@ -292,9 +275,12 @@ class TriggerRuleDep(BaseTIDep):
                         task_ids=ti.task_id, key=PAST_DEPENDS_MET, session=session, default=False
                     )
                     if not past_depends_met:
-                        yield self._failing_status(
-                            reason="Task should be skipped but the past depends are not met"
-                        ), changed
+                        yield (
+                            self._failing_status(
+                                reason="Task should be skipped but the past depends are not met"
+                            ),
+                            changed,
+                        )
                         return
                 changed = ti.set_state(new_state, session)
 
@@ -305,13 +291,16 @@ class TriggerRuleDep(BaseTIDep):
             if ti.map_index > -1:
                 non_successes -= removed
             if non_successes > 0:
-                yield self._failing_status(
-                    reason=(
-                        f"All setup tasks must complete successfully. Relevant setups: {relevant_setups}: "
-                        f"upstream_states={upstream_states}, "
-                        f"upstream_task_ids={task.upstream_task_ids}"
+                yield (
+                    self._failing_status(
+                        reason=(
+                            f"All setup tasks must complete successfully. Relevant setups: {relevant_setups}: "
+                            f"upstream_states={upstream_states}, "
+                            f"upstream_task_ids={task.upstream_task_ids}"
+                        ),
                     ),
-                ), changed
+                    changed,
+                )
 
         def _evaluate_direct_relatives() -> Iterator[TIDepStatus]:
             """Evaluate whether ``ti``'s trigger rule was met.
@@ -540,7 +529,7 @@ class TriggerRuleDep(BaseTIDep):
                             f"upstream_task_ids={task.upstream_task_ids}"
                         )
                     )
-                elif upstream_setup and not success_setup >= 1:
+                elif upstream_setup and not success_setup:
                     yield self._failing_status(
                         reason=(
                             f"Task's trigger rule '{trigger_rule}' requires at least one upstream setup task "
