@@ -88,6 +88,7 @@ _REVISION_HEADS_MAP = {
     "2.6.0": "98ae134e6fff",
     "2.6.2": "c804e5c76e3e",
     "2.7.0": "405de8318b3a",
+    "2.8.0": "10b52ebd31f7",
 }
 
 
@@ -532,14 +533,6 @@ def create_default_connections(session: Session = NEW_SESSION):
         ),
         session,
     )
-    merge_conn(
-        Connection(
-            conn_id="qubole_default",
-            conn_type="qubole",
-            host="localhost",
-        ),
-        session,
-    )
     merge_conn(
         Connection(
             conn_id="redis_default",
@@ -910,7 +903,9 @@ def synchronize_log_template(*, session: Session = NEW_SESSION) -> None:
         select(
             log_template_table.c.filename,
             log_template_table.c.elasticsearch_id,
-        ).order_by(log_template_table.c.id.desc()),
+        )
+        .order_by(log_template_table.c.id.desc())
+        .limit(1)
     ).first()
 
     # If we have an empty table, and the default values exist, we will seed the
@@ -945,6 +940,7 @@ def synchronize_log_template(*, session: Session = NEW_SESSION) -> None:
                 )
             )
             .order_by(log_template_table.c.id.desc())
+            .limit(1)
         ).first()
         if not row:
             session.add(
@@ -1210,9 +1206,8 @@ def _create_table_as(
         )
     else:
         # Postgres and SQLite both support the same "CREATE TABLE a AS SELECT ..." syntax
-        session.execute(
-            f"CREATE TABLE {target_table_name} AS {source_query.selectable.compile(bind=session.get_bind())}"
-        )
+        select_table = source_query.selectable.compile(bind=session.get_bind())
+        session.execute(text(f"CREATE TABLE {target_table_name} AS {select_table}"))
 
 
 def _move_dangling_data_to_new_table(
