@@ -20,9 +20,11 @@ from __future__ import annotations
 from sched import scheduler
 from typing import Callable
 
+from airflow.utils.log.logging_mixin import LoggingMixin
 
-class EventScheduler(scheduler):
-    """General purpose event scheduler"""
+
+class EventScheduler(scheduler, LoggingMixin):
+    """General purpose event scheduler."""
 
     def call_regular_interval(
         self,
@@ -31,9 +33,10 @@ class EventScheduler(scheduler):
         arguments=(),
         kwargs={},
     ):
-        """Helper to call a function at (roughly) a given interval"""
+        """Call a function at (roughly) a given interval."""
 
         def repeat(*args, **kwargs):
+            self.log.debug("Calling %s", action)
             action(*args, **kwargs)
             # This is not perfect. If we want a timer every 60s, but action
             # takes 10s to run, this will run it every 70s.
